@@ -27,7 +27,7 @@ class StoppingProblem:
             config (dict): Configuration
 
         """
-        print("Stand-in for validation")
+        logging.debug("Stand-in for validation")
         # TODO SCHEMAS
 
         errors = validate(config)
