@@ -109,6 +109,13 @@ class ChangeProcess(Module):
 
         cost = self._detection_penalty(action)
 
+        if cost > 0.0:
+            logging.debug(
+                "Got penalty {} for action {} in state {}".format(
+                    cost, action, self._hidden_state
+                )
+            )
+
         new_state = dynamics(self._hidden_state)
 
         if (np.isnan(self._change_time)) and not (
@@ -141,6 +148,9 @@ class ChangeProcess(Module):
             observation_pdfs.append(list(pdf.args))
         yield ("observation_pdfs", observation_pdfs)
 
+    def __repr__(self):
+        return "Change process with state space {}".format(self._state_space)
+
     @property
     def max_duration(self):
         return self._max_duration
@@ -201,12 +211,6 @@ class ChangeProcess(Module):
             action in list(range(self._num_pre_change_states, len(self._state_space)))
         ):
 
-            logging.info(
-                "Result of np.isnan(change_point): {}".format(
-                    np.isnan(self._change_time)
-                )
-            )
-
             if not np.isnan(self._change_time):
 
                 time_since_change = np.float64(
@@ -235,7 +239,7 @@ class ChangeProcess(Module):
 
         elif action == CONTINUE:
 
-            if self._simulation_time == self._max_duration and not np.isnan(
+            if self._simulation_time == self._max_duration - 1 and not np.isnan(
                 self._change_time
             ):
 
@@ -391,13 +395,13 @@ class Threshold(Policy):
 
     @staticmethod
     def _reparameterise(params, **kwargs):
-        """[summary]
+        """Convert decision boundary to free optimisation parameter
 
         Args:
-            params (float): Threshold
+            params (float): Decision boundary (threshold)
 
         Returns:
-            [np.ndarray of float64]: free decision parameter
+            [np.ndarray of float64]: free optimisation parameter parameter
         """
         return logit(params)
 
@@ -417,11 +421,11 @@ class Threshold(Policy):
 class Hyperplanes(Policy):
     def __init__(self, num_states=None, num_pre_change_states=None, planes=None):
 
-        # The constructor needs to initialise a set of valid hyperplanes
         if planes is not None:
             num_post_change_states = np.array(planes).shape[0]
             self._free_params = self._reparameterise(planes)
 
+        # The constructor needs to initialise a set of valid hyperplanes
         else:
 
             if num_states is None:
@@ -435,11 +439,7 @@ class Hyperplanes(Policy):
             param_size = (num_post_change_states, num_states)
 
             self._free_params = uniform(size=param_size)
-            print(
-                "Hyperplanes constructor set params : {}".format(
-                    self._free_params
-                )
-            )
+            print("Hyperplanes constructor set params : {}".format(self._free_params))
         self._num_post_change_states = num_post_change_states
 
     def __call__(self, belief, *args, **kwargs):
@@ -534,6 +534,121 @@ class Hyperplanes(Policy):
         return params
 
 
+class Ellipsoids(Policy):
+    """A decision policy where a change is declared if the belief state enters 
+    the volume of an ellipsoid centred on the vertex of the belief simplex.
+    The number of coefficients per ellipsoid is equal to the ambient dimension of the belief space.
+    """
+
+    def __init__(self, num_states=None, num_pre_change_states=None, ellipsoids=None):
+        if ellipsoids is not None:
+
+            # We need an ellipsoid rule for each post change state
+            num_post_change_states = np.array(ellipsoids).shape[0]
+            self._free_params = self._reparameterise(np.array(ellipsoids))
+
+        else:
+
+            if num_states is None:
+                num_states = 3
+
+            if num_pre_change_states is None:
+                num_pre_change_states = 1
+
+            num_post_change_states = num_states - num_pre_change_states
+
+            if num_post_change_states < 1:
+                raise AttributeError(
+                    "Invalid number of pre- ({}) and post- ({}) change states".format(
+                        num_pre_change_states, num_post_change_states
+                    )
+                )
+
+            param_size = (num_post_change_states, num_states)
+
+            self._free_params = np.ones(size=param_size)
+
+            logging.info(
+                "Ellipsoid constructor set params : {}".format(self._free_params)
+            )
+        self._num_post_change_states = num_post_change_states
+
+    def __call__(self, belief, *args, **kwargs):
+
+        num_states = self._free_params.shape[1]
+
+        for state, ellipsoid in zip(self.post_change_states, self.ellipsoids):
+            test_statistic = 0
+            # For each state, add to the accumulator
+            for i in range(num_states):
+                if i == state:
+                    test_statistic += np.square((belief[i] - 1.0)) / np.square(
+                        ellipsoid[i]
+                    )  # this is the vertex that the ellipsoid is centred on
+                else:
+                    test_statistic += np.square(belief[i]) / np.square(
+                        ellipsoid[i]
+                    )  # just add
+            if test_statistic < 1:
+                logging.debug("Collision detected for state {}".format(state))
+
+                return state
+
+        return CONTINUE
+
+    def __repr__(self):
+        return "Ellipsoids"
+
+    def __iter__(self):
+        yield ("type", self.__repr__())
+        yield ("ellipsoids", [list(e) for e in self.ellipsoids])
+
+    @property
+    def ellipsoids(self):
+        ellipsoids = self._deparameterise(self._free_params)
+        # planes = {}
+        # for state, param in zip(self.post_change_states, params):
+        #     planes[str(state)] = list(param)
+        # return planes
+        return ellipsoids
+
+    @property
+    def post_change_states(self):
+        num_states = self._free_params.shape[
+            1
+        ]  # 0 is number of planes, 1 is belief space dimension
+        first_post_change_state = num_states - self._num_post_change_states
+        return list(range(first_post_change_state, num_states))
+
+    @staticmethod
+    def _reparameterise(ellipsoid_coefficients):
+        """Convert the ellipsoid coefficients to free optimization parameters
+
+        TODO EXPLAIN CHOICE OF REPARAMETERISATION
+
+        Args:
+            ellipsoid_coefficients (numpy.ndarray): MxN where M is the number of anomalies and N is the number of dimensions in belief space.
+            The a, b, c .. etc that appear in the equation for an ellipsoid
+
+        Returns:
+            numpy.ndarray: Optimisation parameters
+        """
+        return expit(ellipsoid_coefficients / np.sqrt(2))
+
+    @staticmethod
+    def _deparameterise(free_parameters):
+        """Convert the free optimisation parameters to the coefficients in an ellipsoid equation
+        Eg in three dimensions x^2/a^2 + y^2/b^2 + z^2/c^2, the ellipsoid coefficients are a, b and c
+
+        Args:
+            free_parameters ([type]): [description]
+
+        Returns:
+            [type]: [description]
+        """
+        return np.sqrt(2) * logit(free_parameters)
+
+
 class Dynamics(Module):
     # TODO COMMON METHODS
 
@@ -697,12 +812,18 @@ class _StoppingProblem(ModuleFactory, FilterMixin):
             change_process["num_pre_change_states"] = 1
 
         self._changeProcess = self._get_module(ChangeProcess, **change_process)
+        logging.debug(
+            "Stopping problem got change process {}".format(self._changeProcess)
+        )
+
         self._dynamics = self._get_module(
             Dynamics,
             **dynamics,
             num_states=change_process["num_states"],
             num_pre_change_states=change_process["num_pre_change_states"]
         )
+
+        logging.debug("Stopping problem got dynamics {}".format(self._dynamics))
         self._policy = self._get_module(
             Policy,
             **policy,
@@ -710,6 +831,8 @@ class _StoppingProblem(ModuleFactory, FilterMixin):
             num_pre_change_states=change_process["num_pre_change_states"]
         )
 
+        logging.debug("Stopping problem got policy {}".format(self._policy))
+
     def __iter__(self):
         yield ("change_process", dict(self._changeProcess))
         yield ("dynamics", dict(self._dynamics))
@@ -718,6 +841,7 @@ class _StoppingProblem(ModuleFactory, FilterMixin):
     def run(self):
 
         self._changeProcess.reset()
+        logging.debug("Reset change process")
 
         filt_hist = np.zeros(
             (self._changeProcess.max_duration, self._changeProcess.num_states),
@@ -730,7 +854,8 @@ class _StoppingProblem(ModuleFactory, FilterMixin):
 
         # Get an initial belief
         belief = self._changeProcess.initial_belief
-
+        logging.debug("Stopping problem got initial belief {}".format(belief))
+        logging.debug("Starting simulation run...")
         while cost == 0 and k < self._changeProcess.max_duration:
 
             observation = self._changeProcess.observe()
