@@ -6,7 +6,7 @@ import numpy as np
 from .lib import (
     Module,
     ChangeProcess,
-    ChangeDetector,
+    Threshold,
     _StoppingProblem,
     AugmentedHMM,
 )
@@ -40,7 +40,7 @@ def adapt_module(module):
 
     if isinstance(module, ChangeProcess):
         return dict(module)
-    elif isinstance(module, ChangeDetector):
+    elif isinstance(module, Threshold):
         return dict(module)
     elif isinstance(module, AugmentedHMM):
         return dict(module)
@@ -78,8 +78,12 @@ def validate(config):
     return []
 
 
-# TODO ADD INFORMATION ABOUT STATES - PRE AND POST 
 class ExperimentSchema(Schema):
+    """Validation schema for experimental results. 
+    This Schema includes all the information necessary for loading serialised data,
+    making meaningful comparisons, plots, etc.
+
+    """
     cost = fields.Float() 
     filter_history = fields.List(fields.List(fields.Float()))
     observation_history = fields.List(fields.Float())
@@ -90,3 +94,10 @@ class ExperimentSchema(Schema):
     timestamp = fields.String()
 
 
+# class OptimiserSchema(Schema):
+#     # A schema that records the information about a particular optimisation.
+#     param_history = fields.List(fields.List(fields.Float()))
+
+
+
+
